unit Ini;

interface

uses
  SysUtils, Classes, Forms;


type
//  PEntry = ^REntry;
//  REntry = record
  TEntry = class(TObject)
  public
    Key, Value : AnsiString;
  end;

  PSection = ^TSection;
  TSection = class(TObject)
  private
    Entries : TList;
    Name : AnsiString;
  public
    constructor Create();
    destructor Destroy(); override;
    function NewEntry(xKey : AnsiString) : TEntry; //PEntry;
    procedure SetName(xName : AnsiString);
    function GetName() : AnsiString;
    function FindIndex(xKey : AnsiString) : Integer;
    function GetValue(xKey : AnsiString; xDefault : AnsiString) : AnsiString;
    procedure SetValue(xKey : AnsiString; xValue : AnsiString);
    procedure WriteText(var xf : TextFile);
  end;

  PIni = ^TIni;
  TIni = class(TObject)
  private
    Sections : TList;
    Name : AnsiString;
  public
    constructor Create();
    destructor Destroy(); override;
    function NewSection(xName : AnsiString) : TSection;
    procedure SetName(xName : AnsiString);
    function GetName() : AnsiString;
    function FindIndex(xSection : AnsiString) : Integer;
    function GetValue(xSection : AnsiString; xKey : AnsiString; xDefault : AnsiString) : AnsiString;
    procedure SetValue(xSection : AnsiString; xKey : AnsiString; xValue : AnsiString);
    procedure WriteIni();

  end;


implementation

uses Windows;

constructor TSection.Create();
begin
  Entries := TList.Create();
end;

destructor TSection.Destroy();
var
 i : Integer;
begin
  for i := 0 to Entries.Count - 1 do
//     FreeMem(Entries.Items[i], SizeOf(REntry));
    TEntry (Entries.Items[i]).Free();
  Entries.Free();
end;

(*
function TSection.NewEntry(xKey : AnsiString) : TEntry; //PEntry;
var
  Entry : PEntry;
begin
  GetMem(Entry, SizeOf(REntry));
  Entries.Add(Entry);
  Entry.Key := xKey;
  Result := Entry;
end;
 *)

function TSection.NewEntry(xKey : AnsiString) : TEntry;
begin
  Result := TEntry.Create();
  Entries.Add(Result);
  Result.Key := xKey;
end;

procedure TSection.SetName(xName : AnsiString);
begin
  Name := xName;
end;

function TSection.GetName() : AnsiString;
begin
  Result := Name;
end;

function TSection.FindIndex(xKey : AnsiString) : Integer;
var
 i : Integer;
begin
  Result := -1;
  for i := 0 to Entries.Count - 1 do
    if AnsiCompareStr(TEntry(Entries.Items[i]).Key, xKey) = 0 then
      Result := i;
end;

function TSection.GetValue(xKey : AnsiString; xDefault : AnsiString) : AnsiString;
var
 i : Integer;
begin
  Result := xDefault;
  i := FindIndex(xKey);
  if i >= 0 then
    Result := TEntry(Entries.Items[i]).Value
//  else
//    NewEntry(xKey).Value := xDefault;
end;

procedure TSection.SetValue(xKey : AnsiString; xValue : AnsiString);
var
 i : Integer;
begin
  i := FindIndex(xKey);
  if i >= 0 then
    TEntry(Entries.Items[i]).Value := xValue
  else
    NewEntry(xKey).Value := xValue;
end;

procedure TSection.WriteText(var xf : TextFile);
var
 i : Integer;
begin
  Write(xf, '[');
  Write(xf, Name);
  WriteLn(xf, ']');

  for i := 0 to Entries.Count - 1 do
  begin
    Write(xf, TEntry(Entries.Items[i]).Key);
    Write(xf, '=');
    WriteLn(xf, TEntry(Entries.Items[i]).Value);
  end;
end;


constructor TIni.Create();
begin
  Sections := TList.Create();
end;

destructor TIni.Destroy();
var
 i : Integer;
begin
  for i := 0 to Sections.Count - 1 do
     TSection (Sections.Items[i]).Free();
  Sections.Free();
end;

function TIni.NewSection(xName : AnsiString) : TSection;
begin
  Result := TSection.Create();
  Sections.Add(Result);
  Result.SetName(xName);
end;

procedure TIni.SetName(xName : AnsiString);
begin
  Name := xName;
end;

function TIni.GetName() : AnsiString;
begin
  Result := Name;
end;

function TIni.FindIndex(xSection : AnsiString) : Integer;
var
 i : Integer;
begin
  Result := -1;
  for i := 0 to Sections.Count - 1 do
    if AnsiCompareStr(TSection(Sections.Items[i]).Name, xSection) = 0 then
      Result := i;
end;

function TIni.GetValue(xSection : AnsiString; xKey : AnsiString; xDefault : AnsiString) : AnsiString;
var
 i : Integer;
begin
  Result := xDefault;
  i := FindIndex(xSection);
  if i >= 0 then
    Result := TSection(Sections.Items[i]).GetValue(xKey, xDefault)
//   else
//    NewSection(xSection).GetValue(xKey, xDefault);
end;

procedure TIni.SetValue(xSection : AnsiString; xKey : AnsiString; xValue : AnsiString);
var
 i : Integer;
begin
  i := FindIndex(xSection);
  if i >= 0 then
    TSection(Sections.Items[i]).SetValue(xKey, xValue)
   else
    NewSection(xSection).SetValue(xKey, xValue);
end;

procedure TIni.WriteIni();
var
 i : Integer;
 f : TextFile;
begin

  AssignFile(f, ChangeFileExt(Application.ExeName, '.ini'));
  Rewrite(f);

  for i := 0 to Sections.Count - 1 do
  begin
    TSection(Sections.Items[i]).WriteText(f);
    WriteLn(f, '');
  end;
  Flush(f);
end;

end.
