unit CustomTransformer;
//----------------------------------------------------------
//   TetraComponents
//   TextTransformer Support
//   Author: Dr. Detlef Meyer-Eltz
//   Many thanks to Dr. Hans-Peter Diettrich for his PASCAL advices
//   http://www.texttransformer.de
//   http://www.texttransformer.com
//   Meyer-Eltz@t-online.de
//
//   March, 2008
//----------------------------------------------------------

interface

uses
  Classes, SysUtils, TetraDll, TetraTypes, TetraMessage;

type
  TOnBeginDocument = procedure (Sender : TObject; const xasName : AnsiString)  of object;
  TOnEnterProduction = procedure (Sender : TObject; const xasName : AnsiString; xeSymno : INTEGER)  of object;
  TOnExit = procedure (Sender : TObject)  of object;
  TOnBranch = procedure (Sender : TObject; const xasName : AnsiString; xeNodeKind : eTetraNodeKind)  of object;
  TOnAcceptToken = procedure (Sender : TObject; const xasName : AnsiString; xeSymno : INTEGER; xlwIgnoredBegin : longword; xlwRecognizedBegin : longword; xlwRecognizedEnd : longword; xbIsSkip : Boolean)  of object;
  TOnExceptionEvent = procedure (Sender : TObject; xasWhat : AnsiString ) of object;
  TOnParserMessage = procedure (Sender : TObject; xasWhat : AnsiString; xlwLastPosition : longword; xlwPosition : longword; const xasProduction : AnsiString; const xasSymbol : AnsiString; xeMessageType : eParserMessage; xlw : longword) of object;
  TOnSourceTarget = procedure (Sender : TObject; const xasSourceFile : AnsiString; const xasTargetFile : AnsiString; const xasMessage : AnsiString) of object;
  //TOnFileError = procedure(Sender: TObject; const xasSourceFile, xasTargetFile, xasMessage: AnsiString) of object;
  TOnFileDone = procedure(Sender: TObject; const xasSourceFile, xasTargetFile, xasMessage: AnsiString; fSucc: boolean) of object;
  TContinue = procedure (Sender : TObject; var xbContinue : BOOLEAN ) of object;

  TCustomTextTransformer = class (TComponent)
  private
    m_bModified : boolean;

    procedure SetEventHandler();

  protected
  //state
    m_bContinue : boolean;
    m_sExtra : AnsiString; //Execute argument
    m_bEnableActions : BOOLEAN;
    m_Messages : TList;
  //params
    m_sProject : TTetraProjectName;
    m_sSourceRoot : AnsiString;
    m_sTargetRoot : AnsiString;
    m_sConfigParam : AnsiString;
    FOnBeginDocument : TOnBeginDocument;
    FOnEndDocument : TOnExit;
    FOnEnterProduction : TOnEnterProduction;
    FOnExitProduction : TOnExit;
    FOnAcceptToken : TOnAcceptToken;
    FOnBeginBranch : TOnBranch;
    FOnEndBranch : TOnExit;
    FOnParserMessage : TOnParserMessage;
    FOnException : TOnExceptionEvent;
    FOnFileError : TOnSourceTarget;
    FOnFileSuccess : TOnSourceTarget;
    FOnFileDone: TOnFileDone;
    FContinue : TContinue;

    function GetModified() : BOOLEAN; virtual;
    procedure SetModified(xb : BOOLEAN); virtual;
    function GetActive() : BOOLEAN;
    function GetStopped() : BOOLEAN;
    procedure SetProject(xasProject : TTetraProjectName);
    procedure SetConfigParam(xasConfigParam : AnsiString );
    procedure SetEnableActions(xbEnableActions : BOOLEAN );
    function ExpandFileName(xasFilename : AnsiString ) : AnsiString;
    procedure FileDone(const xpSourceFile, xpTargetFile, xpMessage: AnsiString; fSucc: boolean);
    procedure AddMessage(xMessage : TTetraMessage );
    function  TransformString(const Source: AnsiString; const ExtraParam: AnsiString = ''): AnsiString;
    function TransformToTree(const Source: AnsiString; const ExtraParam: AnsiString =''): BOOLEAN;
    function  TransformFile(const Source, Target: AnsiString;
      const ExtraParam: AnsiString = ''): BOOLEAN;
    function  TransformToFile(Source: TStrings; const Target: AnsiString;
      const ExtraParam: AnsiString = ''): BOOLEAN;
    function TransformToFiles(Source, Target : TStrings;
      const ExtraParam: AnsiString = ''): BOOLEAN;

    procedure BeforeAcceptToken(const xasName: string; xeSymno : INTEGER; xuiIgnoredBegin, xuiRecognizedBegin, xuiRecognizedEnd: Cardinal; xbIsSkip: Boolean); virtual;
    procedure BeforeEnterProduction(const xasName: string; xeSymno : INTEGER); virtual;
    procedure BeforeExitProduction(); virtual;
    procedure BeforeBeginBranch(const xasName: string; xeNodeKind : eTetraNodeKind); virtual;
    procedure BeforeEndBranch(); virtual;

    property EnableActions : BOOLEAN read m_bEnableActions write SetEnableActions;
    property OnBeginDocument : TOnBeginDocument read FOnBeginDocument  write FOnBeginDocument;
    property OnEndDocument : TOnExit read FOnEndDocument  write FOnEndDocument;
    property OnEnterProduction : TOnEnterProduction read FOnEnterProduction  write FOnEnterProduction;
    property OnExitProduction : TOnExit read FOnExitProduction  write FOnExitProduction;
    property OnAcceptToken : TOnAcceptToken read FOnAcceptToken  write FOnAcceptToken;
    property OnBeginBranch : TOnBranch read FOnBeginBranch  write FOnBeginBranch;
    property OnEndBranch : TOnExit read FOnEndBranch  write FOnEndBranch;
    property OnFileError : TOnSourceTarget read FOnFileError write FOnFileError;
    property OnFileSuccess: TOnSourceTarget read FOnFileSuccess write FOnFileSuccess;
    property OnFileDone: TOnFileDone read FOnFileDone write FOnFileDone;

  public

    constructor Create(Owner : TComponent); override;
    destructor Destroy(); override;

    function IsDllLoaded() : BOOLEAN;
    function IsCompiled() : BOOLEAN;
    function LoadEngine() : BOOLEAN;
    function LoadProject() : BOOLEAN;
    function CompileProject() : BOOLEAN;
    function SetActive() : BOOLEAN; virtual;
    procedure Stop();

    procedure SetSourceRoot(xasSourceRoot : AnsiString );
    procedure SetTargetRoot(xasTargetRoot : AnsiString );

    function GetLastMessage() : TTetraMessage; virtual;
    function GetMessageCount() : INTEGER;
    function GetMessage(xi : INTEGER) : TTetraMessage;
    function HasError() : BOOLEAN;
    procedure ClearMessages();

    function GetDllVersionMajor() : INTEGER;
    function GetDllVersionMinor() : INTEGER;
    function GetDllVersionRelease() : INTEGER;
    function GetDllVersionString() : AnsiString;
    function GetComponentVersionString() : AnsiString;

    function Execute(const xasExtraParam : AnsiString = '') : BOOLEAN; virtual;
    procedure FreeDll();
    procedure FreeLibrary();
  published

    // in
    property Project : TTetraProjectName read m_sProject  write SetProject;
    property ConfigParam: AnsiString read m_sConfigParam write SetConfigParam;

    // read only
    property Active : BOOLEAN read GetActive;
    property Modified : BOOLEAN read GetModified;
    property Stopped : BOOLEAN read GetStopped;

    // events
    property OnException : TOnExceptionEvent read FOnException  write FOnException;
    property OnParserMessage : TOnParserMessage read FOnParserMessage  write FOnParserMessage;
    property Continue : TContinue read FContinue  write FContinue;
  end;

//---------------------------------------------------------------------------

implementation

uses
  Forms, Windows, Dialogs;

var
  _pTTSingleton : TCustomTextTransformer;
  _sCompiledProject: TTetraProjectName;
  _bActionsEnabled: BOOLEAN;

//---------------------------------------------------------------------------

procedure _OnParserMessage( const xpWhat : PAnsiChar;
                            xlwLastPosition : longword;
                            xlwPosition : longword;
                            const xpProduction : PAnsiChar;
                            const xpSymbol : PAnsiChar;
                            xeMessageType : eParserMessage;
                            xlw : longword) cdecl;
begin
  if (_pTTSingleton <> NIL) and Assigned(_pTTSingleton.OnParserMessage) then
  begin
    _pTTSingleton.OnParserMessage(_pTTSingleton,
                                  xpWhat,
                                  xlwLastPosition,
                                  xlwPosition,
                                  xpProduction,
                                  xpSymbol,
                                  xeMessageType,
                                  xlw);

  if xeMessageType in [pmExit, pmError, pmParseError, pmSemError] then
     _pTTSingleton.AddMessage(TTetraParserMessage.Make(_pTTSingleton,
                                                       xpWhat,
                                                       xlwLastPosition,
                                                       xlwPosition,
                                                       xpProduction,
                                                       xpSymbol,
                                                       true ))
  else
     _pTTSingleton.AddMessage(TTetraParserMessage.Make(_pTTSingleton,
                                                       xpWhat,
                                                       xlwLastPosition,
                                                       xlwPosition,
                                                       xpProduction,
                                                       xpSymbol,
                                                       false ))

  end;
end;

//---------------------------------------------------------------------------
procedure _OnBeginDocument(const xpName : PAnsiChar) cdecl;
begin
  if _pTTSingleton <> NIL then
  begin
     if Assigned(_pTTSingleton.OnBeginDocument) then
     _pTTSingleton.OnBeginDocument(_pTTSingleton, xpName);
  end;
end;
//---------------------------------------------------------------------------
procedure _OnEndDocument() cdecl;
begin
  if _pTTSingleton <> NIL then
  begin
     if Assigned(_pTTSingleton.OnEndDocument) then
     _pTTSingleton.OnEndDocument(_pTTSingleton);
  end;
end;

procedure _OnEnterProduction(const xpName : PAnsiChar; xeSymno : INTEGER) cdecl;
begin
  if _pTTSingleton <> NIL then begin
    _pTTSingleton.BeforeEnterProduction(xpName, xeSymno);
    if Assigned(_pTTSingleton.OnEnterProduction) then
       _pTTSingleton.OnEnterProduction(_pTTSingleton, xpName, xeSymno);
  end;
end;

procedure _OnExitProduction() cdecl;
begin
  if _pTTSingleton <> NIL then begin
    _pTTSingleton.BeforeExitProduction();
    if Assigned(_pTTSingleton.OnExitProduction) then
       _pTTSingleton.OnExitProduction(_pTTSingleton);
  end;
end;

procedure _OnAcceptToken(const xpName: PAnsiChar; xeSymno : INTEGER;
  xuiIgnoredBegin, xuiRecognizedBegin, xuiRecognizedEnd: longword;
  xbIsSkip: Boolean) cdecl;
begin
  if _pTTSingleton <> NIL then begin
    _pTTSingleton.BeforeAcceptToken(xpName, xeSymno, xuiIgnoredBegin + 1, xuiRecognizedBegin + 1, xuiRecognizedEnd + 1, xbIsSkip);
    if Assigned(_pTTSingleton.OnAcceptToken) then
       _pTTSingleton.OnAcceptToken(_pTTSingleton, xpName, xeSymno, xuiIgnoredBegin + 1, xuiRecognizedBegin + 1, xuiRecognizedEnd + 1, xbIsSkip);
  end;
end;

//---------------------------------------------------------------------------
procedure _OnBeginBranch(const xpName : PAnsiChar; xeNodeKind : eTetraNodeKind) cdecl;
begin
  if _pTTSingleton <> NIL then begin
    _pTTSingleton.BeforeBeginBranch(xpName, xeNodeKind);
    if Assigned(_pTTSingleton.OnBeginBranch) then
       _pTTSingleton.OnBeginBranch(_pTTSingleton, xpName, xeNodeKind);
  end;
end;
//---------------------------------------------------------------------------
procedure _OnEndBranch() cdecl;
begin
  if _pTTSingleton <> NIL then begin
    _pTTSingleton.BeforeEndBranch();
    if Assigned(_pTTSingleton.OnEndBranch) then
       _pTTSingleton.OnEndBranch(_pTTSingleton);
  end;
end;
//---------------------------------------------------------------------------
procedure _OnException(const xpWhat : PAnsiChar) cdecl;
begin
  if (_pTTSingleton <> NIL) and Assigned(_pTTSingleton.OnException) then
  begin
     _pTTSingleton.OnException(_pTTSingleton, xpWhat);
     _pTTSingleton.AddMessage(TTetraError.Make(_pTTSingleton, xpWhat));
  end;
end;
//---------------------------------------------------------------------------
procedure _OnFileError(const xpSourceFile : PAnsiChar; const xpTargetFile : PAnsiChar; const xpMessage : PAnsiChar) cdecl;
begin
  if (_pTTSingleton <> NIL) and Assigned(_pTTSingleton.OnFileError) then
  begin
     _pTTSingleton.OnFileError(_pTTSingleton, xpSourceFile, xpTargetFile, xpMessage);
     _pTTSingleton.AddMessage(TTetraFileMessage.Make(_pTTSingleton,
                                                     xpMessage,
                                                     xpSourceFile,
                                                     xpTargetFile,
                                                     true));
  end;
end;
//---------------------------------------------------------------------------
function _TestEscapePressed() : BOOLEAN;
begin
  Application.ProcessMessages();
  if Application.Active then
     result := (0 = GetAsyncKeyState( VK_ESCAPE ))
  else
    begin
     GetAsyncKeyState( VK_ESCAPE );
     result := true;
    end;
end;
//---------------------------------------------------------------------------
function _Continue() : BOOLEAN cdecl;
var
  bContinue : BOOLEAN;
begin
  if _pTTSingleton <> NIL then
  begin
    result := not _pTTSingleton.GetStopped();
    if result then
    begin
      if Assigned(_pTTSingleton.Continue) then
      begin
        bContinue := true;
        _pTTSingleton.Continue(_pTTSingleton, bContinue);
        result := bContinue;
      end
    else
      result:= _TestEscapePressed();
    end;
    if not result then
      _pTTSingleton.Stop();
  end
  else
   result:= true;

  //if not result then
  //  Application.MessageBox('user break', '');
end;

//---------------------------------------------------------------------------

{ TCustomTextTransformer }

constructor TCustomTextTransformer.Create(Owner : TComponent);
begin
  inherited Create(Owner);
  FOnParserMessage := NIL;
  FOnBeginDocument := NIL;
  FOnEndDocument := NIL;
  FOnEnterProduction := NIL;
  FOnExitProduction := NIL;
  FOnAcceptToken := NIL;
  FOnException := NIL;
  FOnFileError := NIL;
  FOnFileSuccess := NIL;
  FContinue := NIL;
  SetModified(false);
  m_bEnableActions := true;
  m_Messages := TList.Create();
end;
//---------------------------------------------------------------------------
destructor TCustomTextTransformer.Destroy();
begin
  FreeAndNil(m_Messages);
  inherited Destroy;
end;
//---------------------------------------------------------------------------
procedure TCustomTextTransformer.FreeDll();
begin
  _TetraDll.FreeDll();
  _pTTSingleton := NIL;
  _sCompiledProject := '';
  _bActionsEnabled := true;
end;
//---------------------------------------------------------------------------
procedure TCustomTextTransformer.FreeLibrary();
begin
  _TetraDll.FreeLibrary();
end;
//---------------------------------------------------------------------------
function TCustomTextTransformer.GetDllVersionMajor() : INTEGER;
begin
  if _TetraDll.IsDllLoaded then
    result := _TetraDll._GetVersionMajor()
   else
    result := 0;
end;
//---------------------------------------------------------------------------
function TCustomTextTransformer.GetDllVersionMinor() : INTEGER;
begin
  if _TetraDll.IsDllLoaded = true then
    result := _TetraDll._GetVersionMinor()
   else
    result := 0;
end;
//---------------------------------------------------------------------------
function TCustomTextTransformer.GetDllVersionRelease() : INTEGER;
begin
  if _TetraDll.IsDllLoaded = true then
    result := _TetraDll._GetVersionRelease()
   else
    result := 0;
end;
//---------------------------------------------------------------------------
function TCustomTextTransformer.GetDllVersionString() : AnsiString;
begin
   result := _TetraDll.GetDllVersionString();
end;
//---------------------------------------------------------------------------
function TCustomTextTransformer.GetComponentVersionString() : AnsiString;
begin
  Result := _TetraDll.GetDllVersionString();
end;
//---------------------------------------------------------------------------
function TCustomTextTransformer.IsDllLoaded() : BOOLEAN;
begin
  Result := _TetraDll.IsDllLoaded;
end;
//---------------------------------------------------------------------------
function TCustomTextTransformer.IsCompiled() : BOOLEAN;
begin
  result :=  (AnsiCompareText(_sCompiledProject, m_sProject) = 0) and
             (_bActionsEnabled = m_bEnableActions);
end;
//---------------------------------------------------------------------------
function TCustomTextTransformer.GetActive() : BOOLEAN;
begin
  Result := (_pTTSingleton = self) and IsCompiled();
end;
//---------------------------------------------------------------------------
procedure TCustomTextTransformer.SetEventHandler();
begin
  _TetraDll._SetOnException(_OnException );
  _TetraDll._SetOnFileError(_OnFileError );
  _TetraDll._SetContinueFn(_Continue );

  _TetraDll._SetOnBeginDocument(_OnBeginDocument );
  _TetraDll._SetOnEndDocument(_OnEndDocument );
  _TetraDll._SetOnEnterProduction(_OnEnterProduction );
  _TetraDll._SetOnExitProduction(_OnExitProduction );
  _TetraDll._SetOnAcceptToken(_OnAcceptToken );
  _TetraDll._SetOnBeginBranch(_OnBeginBranch );
  _TetraDll._SetOnEndBranch(_OnEndBranch );

  _TetraDll._SetOnParserMessage(_OnParserMessage);
  _TetraDll._SetOnFileError(_OnFileError );
  _TetraDll._SetSourceRoot(PAnsiChar(m_sSourceRoot));
  _TetraDll._SetTargetRoot(PAnsiChar(m_sTargetRoot));
  _TetraDll._SetConfigParam(PAnsiChar(m_sConfigParam));

end;
//---------------------------------------------------------------------------
function TCustomTextTransformer.LoadEngine() : BOOLEAN;
begin
  _pTTSingleton := NIL;
  Result := _TetraDll.IsDllLoaded;
  if Result then
    EXIT;

  if _TetraDll.LoadEngine() then begin
    Result := true;
    SetEventHandler();
  end
  else
    ShowMessage('could not load "tetra_engine.dll"');

end;
//---------------------------------------------------------------------------
function TCustomTextTransformer.LoadProject() : BOOLEAN;
begin
  Result := LoadEngine();
  if not Result then
    EXIT;

  _pTTSingleton := self;  // even if not compiled, _pTTSingleton transports the information for the event handlers

  if not IsCompiled then
    result := _TetraDll._LoadProject(PAnsiChar(m_sProject));
end;
//---------------------------------------------------------------------------
function TCustomTextTransformer.CompileProject() : BOOLEAN;
begin
  Result := LoadProject();
  if not Result then
    EXIT;

  if IsCompiled then
    EXIT;

  _bActionsEnabled := m_bEnableActions;
  _TetraDll._EnableActions(m_bEnableActions);
  m_bContinue := true;  // otherwise the continuation test may fail
  _sCompiledProject := m_sProject;  // to be set, even if copilation fails, to force new compilation, if another component is activated
  Result := _TetraDll._CompileProject;
  if not Result then begin
    _pTTSingleton := NIL;
    _sCompiledProject := '';
  end;
end;
//---------------------------------------------------------------------------
function TCustomTextTransformer.SetActive(): boolean;
begin
  ClearMessages();
  if not GetModified() and GetActive() then begin
    Result := true;
    Exit;
  end;

(*  -> make manual check according to your needs
    if not _TetraDll.CheckVersion() then
    begin
      ShowMessage( '"tetra_engine.dll" is out of date\r\n' +
                   'tetra_enginge.dll version : ' + _TetraDll.GetDllVersionString() + '\r\n' +
                   'Component version         : ' + _TetraDll.GetComponentVersionString());
      EXIT;
    end;
 *)

  Result := false;
  if not CompileProject() then
     EXIT;

  SetModified(true);
  Result := _TetraDll.SetActive();
  if Result then
  begin
    _pTTSingleton := self;
    m_bContinue := true;
  end;
end;
//---------------------------------------------------------------------------
function TCustomTextTransformer.Execute(const xasExtraParam : AnsiString ) : BOOLEAN;
begin
  result := true;  // should be overwritten
end;
//---------------------------------------------------------------------------
function TCustomTextTransformer.GetLastMessage() : TTetraMessage;
begin
  result := m_Messages.Last;
end;
//---------------------------------------------------------------------------
function TCustomTextTransformer.GetMessageCount() : INTEGER;
begin
  result := m_Messages.Count;
end;
//---------------------------------------------------------------------------
function TCustomTextTransformer.GetMessage(xi : INTEGER) : TTetraMessage;
begin
  result := NIL;
  if xi < m_Messages.Count then
     result := m_Messages[xi];
end;
//---------------------------------------------------------------------------
function TCustomTextTransformer.HasError() : BOOLEAN;
var
 i : INTEGER;
begin
  result := false;
  for i := 0 to m_Messages.Count - 1 do
    if TTetraMessage(m_Messages.Items[i]).IsError() then
    begin
      result := true;
      break;
    end;
end;
//---------------------------------------------------------------------------
procedure TCustomTextTransformer.ClearMessages();
begin
  AddMessage(TTetraMessage.Make(self, 'ok', false));
end;
//---------------------------------------------------------------------------
procedure TCustomTextTransformer.SetSourceRoot(xasSourceRoot : AnsiString);
begin
{ xasSourceRoot is copied inside of the dll }
  if AnsiCompareText(m_sSourceRoot, xasSourceRoot) <> 0 then begin
    SetModified(true);
    m_sSourceRoot := xasSourceRoot;
  end;
  if GetActive() then
    _TetraDll._SetSourceRoot(PAnsiChar(xasSourceRoot));
end;
//---------------------------------------------------------------------------
procedure TCustomTextTransformer.SetTargetRoot(xasTargetRoot : AnsiString);
begin
{ xasTargetRoot is copied inside of the dll }

  if AnsiCompareText(m_sTargetRoot, xasTargetRoot) <> 0 then begin
    SetModified(true);
    m_sTargetRoot := xasTargetRoot;
  end;

  if GetActive() then
    _TetraDll._SetTargetRoot(PAnsiChar(xasTargetRoot));
end;
//---------------------------------------------------------------------------
procedure TCustomTextTransformer.SetConfigParam(xasConfigParam : AnsiString);
begin
  if AnsiCompareText(m_sConfigParam, xasConfigParam) <> 0 then begin
    SetModified(true);
    m_sConfigParam := xasConfigParam;
  end;

  if GetActive() then
    _TetraDll._SetConfigParam(PAnsiChar(xasConfigParam));
end;
//---------------------------------------------------------------------------
procedure TCustomTextTransformer.SetProject(xasProject : TTetraProjectName);
begin
  if( AnsiCompareText(m_sProject, xasProject) <> 0) then begin
    SetModified(true);
    m_sProject := xasProject;
  end;
end;
//---------------------------------------------------------------------------
procedure TCustomTextTransformer.SetEnableActions(xbEnableActions : BOOLEAN );
begin
  if xbEnableActions <> m_bEnableActions then begin
    m_bEnableActions := xbEnableActions;
    SetModified(true);
  end;
end;
//---------------------------------------------------------------------------
procedure TCustomTextTransformer.AddMessage(xMessage : TTetraMessage);
begin
   m_Messages.Add(xMessage);
end;
//---------------------------------------------------------------------------
function TCustomTextTransformer.ExpandFileName(xasFilename : AnsiString) : AnsiString;
begin
  if ExtractFileDrive(xasFilename) <> '' then
     result := xasFilename
  else begin
    Result := m_sSourceRoot + '\' + xasFilename;
  end;
end;
//---------------------------------------------------------------------------
procedure TCustomTextTransformer.Stop();
begin
  m_bContinue := false;
end;
//---------------------------------------------------------------------------
function TCustomTextTransformer.GetStopped() : BOOLEAN;
begin
  result := not m_bContinue;
end;
//---------------------------------------------------------------------------
function TCustomTextTransformer.GetModified() : BOOLEAN;
begin
  result := m_bModified or not GetActive();
end;
//---------------------------------------------------------------------------
procedure TCustomTextTransformer.SetModified(xb : BOOLEAN);
begin
  if (csLoading in ComponentState) or (csDesigning	in ComponentState) then
    EXIT;

  m_bModified := xb;
end;
// ------------------ functions --------------------
function TCustomTextTransformer.TransformString(const Source: AnsiString; const ExtraParam: AnsiString = ''): AnsiString;
begin
  if SetActive() then begin
    m_sExtra := ExtraParam;
    Result := _TetraDll._TransformTextToText(PAnsiChar(Source), PAnsiChar(ExtraParam));
    SetModified(false);
  end else
    Result := '';
end;

function TCustomTextTransformer.TransformToTree(const Source: AnsiString; const ExtraParam: AnsiString = ''): BOOLEAN;
begin
  if SetActive() then begin
    m_sExtra := ExtraParam;
    Result:= _TetraDll._TransformTextToTree(PAnsiChar(Source), PAnsiChar(ExtraParam));
    SetModified(false);
  end else
    Result := false;
end;

function TCustomTextTransformer.TransformFile(const Source, Target: AnsiString;
  const ExtraParam: AnsiString = ''): boolean;
var
  asSourceFile: AnsiString;
begin
  Result := SetActive();
  if not Result then
    exit;

  m_sExtra := ExtraParam;
  asSourceFile := ExpandFileName(Source);
  Result := FileExists(asSourceFile);
  if not Result then
  begin
    FileDone(asSourceFile, Target, 'source file does not exist', False);
    EXIT;
  end;
    _TetraDll._TransformToFileOne2One(PAnsiChar(asSourceFile), PAnsiChar(Target), PAnsiChar(ExtraParam));
  Result := not HasError();
end;

function TCustomTextTransformer.TransformToFile(Source: TStrings;
  const Target: AnsiString;
  const ExtraParam: AnsiString = ''): boolean;
var
  asSourceFile, asTargetFile: AnsiString;
  i, iSourceFileCount: integer;
begin
  Result := SetActive();
  if not Result then
    Exit;

  m_sExtra := ExtraParam;
  asTargetFile := ExpandFileName(Target);
  iSourceFileCount := Source.Count - 1;

  for  i := 0 to iSourceFileCount do begin
    asSourceFile := ExpandFileName(Source[i]);
    Result := FileExists(asSourceFile);
    if Result then begin
      Result := _TetraDll._TransformToFileN2One(PAnsiChar(asSourceFile), PAnsiChar(Target), i = 0 , i = iSourceFileCount, PAnsiChar(ExtraParam));
      FileDone(asSourceFile, Target, asSourceFile + ' transformed to ' + Target, Result);
    end else begin
      FileDone(asSourceFile, Target, 'source file does not exist', False);
    end;

    if not m_bContinue then
       break;
  end;
end;

function TCustomTextTransformer.TransformToFiles(Source, Target : TStrings;
  const ExtraParam: AnsiString = ''): boolean;
var
  iSourceFileCount : INTEGER;
  iTargetFileCount : INTEGER;
  i : INTEGER;
  asSourceFile : AnsiString;
begin
  Result := SetActive();
  if not Result then
    Exit;

  m_sExtra := ExtraParam;
  iSourceFileCount := Source.Count;
  iTargetFileCount := Target.Count;

  Result := iSourceFileCount = iTargetFileCount;
  if Result then begin // N : N
    for i := 0 to iSourceFileCount - 1 do begin
      Result := TransformFile(Source[i], Target[i], ExtraParam);
      FileDone(asSourceFile, Target[i], asSourceFile + ' to ' + Target[i], Result);

      if not m_bContinue then
         break;
    end
  end else begin
    if Assigned(FOnException) then
       FOnException(self, 'different numbers of source files and target files');
  end;

end;


procedure TCustomTextTransformer.FileDone(const xpSourceFile, xpTargetFile,
  xpMessage: AnsiString; fSucc: boolean);
begin
  AddMessage(TTetraError.Make(self, xpMessage));
  if Assigned(FOnFileDone) then
     FOnFileDone(self, xpSourceFile, xpTargetFile, xpMessage, fSucc)
  else if fSucc then begin
    if Assigned(OnFileSuccess) then
      FOnFileSuccess(self, xpSourceFile, xpTargetFile, xpMessage);
  end else if Assigned(OnFileError) then
    FOnFileError(self, xpSourceFile, xpTargetFile, xpMessage);
end;

// ------------------ tree production --------------------

procedure TCustomTextTransformer.BeforeAcceptToken(const xasName: string; xeSymno : INTEGER;
  xuiIgnoredBegin, xuiRecognizedBegin, xuiRecognizedEnd: Cardinal;
  xbIsSkip: Boolean);
begin
///purpose: create token node
  //to be overwritten
end;

procedure TCustomTextTransformer.BeforeEnterProduction(const xasName: string; xeSymno : INTEGER);
begin
///purpose: create production node
  //to be overwritten
end;

procedure TCustomTextTransformer.BeforeExitProduction();
begin
///purpose: reactivate parent node
  //to be overwritten
end;

procedure TCustomTextTransformer.BeforeBeginBranch(const xasName: string; xeNodeKind : eTetraNodeKind);
begin
///purpose: create production node
  //to be overwritten
end;

procedure TCustomTextTransformer.BeforeEndBranch();
begin
///purpose: reactivate parent node
  //to be overwritten
end;


initialization
  _pTTSingleton := NIL;
  _bActionsEnabled := true;
end.
