unit TetraMessage;

interface

uses
Classes, TetraTypes;

type

  PMessage = ^TTetraMessage;
  TTetraMessage = class (TComponent)
  public
    constructor Make(Owner : TComponent; xasMessage : AnsiString; bIsError : BOOLEAN); 
    function IsError() : BOOLEAN;
    function What() : AnsiString;
    function MessageType() : eTetraMessageKind; virtual;

  private
    m_asMessage : AnsiString;
    m_bIsError : BOOLEAN;
  end;

  TTetraError = class (TTetraMessage)
  public
    constructor Make(Owner : TComponent; xasMessage : AnsiString); virtual;
    function MessageType() : eTetraMessageKind; override;
  end;

  TTetraFileMessage = class (TTetraMessage)
  public
    constructor Make(Owner : TComponent;
                       xasMessage : AnsiString;
                       xasSourceFile : AnsiString;
                       xasTargetFile : AnsiString;
                       xbIsError : BOOLEAN);
    function MessageType() : eTetraMessageKind; override;
    function SourceFile() : AnsiString;
    function TargetFile() : AnsiString;

  private
    m_asSourceFile : AnsiString;
    m_asTargetFile : AnsiString;
  end;

  TTetraParserMessage = class (TTetraMessage)
  public
    m_lwLastPosition : longword;
    m_lwPosition : longword;
    m_asProduction : AnsiString;
    m_asSymbol : AnsiString;

    constructor Make(Owner : TComponent;
                       xasMessage : AnsiString;
                       xlwLastPosition : longword;
                       xlwPosition : longword;
                       xasProduction : AnsiString;
                       xasSymbol : AnsiString;
                       xbIsError : BOOLEAN);
    function MessageType() : eTetraMessageKind; override;

  end;

implementation

constructor TTetraMessage.Make(Owner : TComponent; xasMessage : AnsiString; bIsError : BOOLEAN);
begin
  inherited Create(Owner);
  m_asMessage := xasMessage;
  m_bIsError := bIsError;
end;

function TTetraMessage.MessageType() : eTetraMessageKind;
begin
  Result := tmkMessage;
end;

function TTetraMessage.IsError() : BOOLEAN;
begin
  Result := m_bIsError;
end;

function TTetraMessage.What() : AnsiString;
begin
  Result := m_asMessage;
end;


constructor TTetraError.Make(Owner : TComponent; xasMessage : AnsiString);
begin
  inherited Make(Owner, xasMessage , true);
end;

function TTetraError.MessageType() : eTetraMessageKind;
begin
  Result := tmkError;
end;


constructor TTetraFileMessage.Make(Owner : TComponent;
                                   xasMessage : AnsiString;
                                   xasSourceFile : AnsiString;
                                   xasTargetFile : AnsiString;
                                   xbIsError : BOOLEAN);
begin
  inherited Make(Owner, xasMessage, xbIsError);
end;

function TTetraFileMessage.SourceFile() : AnsiString;
begin
  Result := m_asSourceFile;
end;

function TTetraFileMessage.TargetFile() : AnsiString;
begin
  Result := m_asTargetFile;
end;


function TTetraFileMessage.MessageType() : eTetraMessageKind;
begin
  Result := tmkFileMessage;
end;

constructor TTetraParserMessage.Make(Owner : TComponent;
                                       xasMessage : AnsiString;
                                       xlwLastPosition : longword;
                                       xlwPosition : longword;
                                       xasProduction : AnsiString;
                                       xasSymbol : AnsiString;
                                       xbIsError : BOOLEAN);
begin
  inherited Make(Owner, xasMessage, xbIsError);

  m_lwLastPosition := xlwLastPosition;
  m_lwPosition := xlwPosition;
  m_asProduction := xasProduction;
  m_asSymbol := xasSymbol;

end;

function TTetraParserMessage.MessageType() : eTetraMessageKind;
begin
  Result := tmkParserMessage;
end;

end.



